@@ -38,16 +38,14 @@ from gi.repository import Adw, Gtk, Gio, GLib, Gdk, GObject
 from gnomemusic.about import show_about
 from gnomemusic.coregrilo import CoreGrilo
 from gnomemusic.coremodel import CoreModel
-from gnomemusic.coreselection import CoreSelection
 from gnomemusic.inhibitsuspend import InhibitSuspend
 from gnomemusic.mpris import MPRIS
 from gnomemusic.musiclogger import MusicLogger
 from gnomemusic.notificationmanager import NotificationManager
 from gnomemusic.pauseonsuspend import PauseOnSuspend
 from gnomemusic.player import Player
-from gnomemusic.scrobbler import LastFmScrobbler
 from gnomemusic.search import Search
-from gnomemusic.widgets.lastfmdialog import LastfmDialog
+from gnomemusic.widgets.preferencesdialog import PreferencesDialog
 from gnomemusic.window import Window
 
 
@@ -69,18 +67,14 @@ class Application(Adw.Application):
         self._search = Search()
 
         self._notificationmanager = NotificationManager(self)
-        self._coreselection = CoreSelection()
         self._coremodel = CoreModel(self)
         # Order is important: CoreGrilo initializes the Grilo sources,
-        # which in turn use CoreModel & CoreSelection extensively.
+        # which in turn uses CoreModel extensively.
         self._coregrilo = CoreGrilo(self)
 
         self._settings = Gio.Settings.new('org.gnome.Music')
-        self._lastfm_scrobbler = LastFmScrobbler(self)
         self._player = Player(self)
 
-        self._lastfm_dialog: Optional[LastfmDialog] = None
-
         InhibitSuspend(self)
         PauseOnSuspend(self._player)
 
@@ -134,25 +128,6 @@ class Application(Adw.Application):
         """
         return self._coremodel
 
-    @GObject.Property(
-        type=CoreSelection, flags=GObject.ParamFlags.READABLE)
-    def coreselection(self):
-        """Get selection object.
-
-        :returns: Object containing all selection info
-        :rtype: CoreSelection
-        """
-        return self._coreselection
-
-    @GObject.Property(type=LastFmScrobbler, flags=GObject.ParamFlags.READABLE)
-    def lastfm_scrobbler(self):
-        """Get Last.fm scrobbler.
-
-        :returns: Last.fm scrobbler
-        :rtype: LastFmScrobbler
-        """
-        return self._lastfm_scrobbler
-
     @GObject.Property(type=Window, flags=GObject.ParamFlags.READABLE)
     def window(self):
         """Get main window.
@@ -184,9 +159,10 @@ class Application(Adw.Application):
 
     def _set_actions(self):
         action_entries = [
-            ('about', self._about, None),
+            ("about", self._about, None),
+            ("preferences", self._preferences_dialog,
+                ("app.preferences", ["<Ctrl>comma"])),
             ("help", self._help, ("app.help", ["F1"])),
-            ("lastfm-configure", self._lastfm_account, None),
             ("quit", self._quit, ("app.quit", ["<Ctrl>Q"]))
         ]
 
@@ -209,21 +185,11 @@ class Application(Adw.Application):
             self._window, "help:gnome-music", Gdk.CURRENT_TIME, None,
             show_uri_cb)
 
-    def _lastfm_account(
+    def _preferences_dialog(
             self, action: Gio.SimpleAction,
-            param: Optional[GLib.Variant]) -> None:
-
-        def on_response(dialog: LastfmDialog, response_id: int) -> None:
-            if not self._lastfm_dialog:
-                return
-
-            self._lastfm_dialog.destroy()
-            self._lastfm_dialog = None
-
-        self._lastfm_dialog = LastfmDialog(
-            self._window, self._lastfm_scrobbler)
-        self._lastfm_dialog.connect("response", on_response)
-        self._lastfm_dialog.present()
+            param_type: GLib.VariantType) -> None:
+        pref_dialog = PreferencesDialog(self)
+        pref_dialog.present(self._window)
 
     def _about(self, action, param):
         show_about(self.props.application_id, self._version, self._window)
