@@ -27,6 +27,7 @@ gi.require_versions({"Grl": "0.3"})
 from gi.repository import Grl, Gtk, GObject
 
 from gnomemusic.artistart import ArtistArt
+from gnomemusic.corealbum import CoreAlbum
 import gnomemusic.utils as utils
 
 
@@ -49,7 +50,6 @@ class CoreArtist(GObject.GObject):
         self._coregrilo = application.props.coregrilo
         self._coremodel = application.props.coremodel
         self._model = None
-        self._selected = False
         self._thumbnail = None
 
         self.update(media)
@@ -63,60 +63,23 @@ class CoreArtist(GObject.GObject):
             self._coremodel.props.albums)
         albums_model_filter.set_filter(Gtk.AnyFilter())
 
-        albums_model_sort = Gtk.SortListModel.new(albums_model_filter)
+        albums_sort_exp = Gtk.PropertyExpression.new(CoreAlbum, None, "year")
+        albums_sorter = Gtk.StringSorter.new(albums_sort_exp)
+        albums_model_sort = Gtk.SortListModel.new(
+            albums_model_filter, albums_sorter)
 
         self._coregrilo.get_artist_albums(
             self.props.media, albums_model_filter)
 
-        def _album_sort(album_a, album_b, data=None):
-            year_a = album_a.props.year
-            year_b = album_b.props.year
-            if None in [year_a, year_b]:
-                return Gtk.Ordering.EQUAL
-            elif year_a > year_b:
-                return Gtk.Ordering.LARGER
-            elif year_a < year_b:
-                return Gtk.Ordering.SMALLER
-            else:
-                return Gtk.Ordering.EQUAL
-
-        albums_sorter = Gtk.CustomSorter()
-        albums_sorter.set_sort_func(_album_sort)
-        albums_model_sort.set_sorter(albums_sorter)
-
         return albums_model_sort
 
     @GObject.Property(type=Gtk.SortListModel, default=None)
     def model(self):
         if self._model is None:
             self._model = self._get_artist_album_model()
-            self._model.connect("items-changed", self._on_items_changed)
 
         return self._model
 
-    def _on_items_changed(self, model, pos, removed, added):
-        with self.freeze_notify():
-            for corealbum in self._model:
-                corealbum.props.selected = self.props.selected
-
-    @GObject.Property(type=bool, default=False)
-    def selected(self):
-        return self._selected
-
-    @selected.setter  # type: ignore
-    def selected(self, value):
-        if value == self._selected:
-            return
-
-        self._selected = value
-
-        # The model is loaded on-demand, so the first time the model is
-        # returned it can still be empty. This is problem for returning
-        # a selection. Trigger loading of the model here if a selection
-        # is requested, it will trigger the filled model update as
-        # well.
-        self.props.model.items_changed(0, 0, 0)
-
     @GObject.Property(type=str, default=None)
     def thumbnail(self):
         """Artist art thumbnail retrieval
