@@ -22,9 +22,9 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-from gi.repository import GObject, Gio, Gfm, Grl
+from gi.repository import GObject, Grl, Gtk
 
-import gnomemusic.utils as utils
+from gnomemusic.coresong import CoreSong
 
 
 class CoreDisc(GObject.GObject):
@@ -46,11 +46,8 @@ class CoreDisc(GObject.GObject):
 
         self._coregrilo = application.props.coregrilo
         self._coremodel = application.props.coremodel
-        self._filter_model = None
         self._log = application.props.log
         self._model = None
-        self._old_album_ids = []
-        self._selected = False
 
         self.update(media)
         self.props.disc_nr = nr
@@ -58,23 +55,21 @@ class CoreDisc(GObject.GObject):
     def update(self, media):
         self.props.media = media
 
-    @GObject.Property(type=Gio.ListModel, default=None)
-    def model(self):
-        def _disc_sort(song_a, song_b):
-            return song_a.props.track_number - song_b.props.track_number
-
+    @GObject.Property(type=Gtk.SortListModel, default=None)
+    def model(self) -> Gtk.SortListModel:
         if self._model is None:
-            self._filter_model = Gfm.FilterListModel.new(
+            filter_model = Gtk.FilterListModel.new(
                 self._coremodel.props.songs)
-            self._filter_model.set_filter_func(lambda a: False)
-            self._model = Gfm.SortListModel.new(self._filter_model)
-            self._model.set_sort_func(
-                utils.wrap_list_store_sort_func(_disc_sort))
+            filter_model.set_filter(Gtk.AnyFilter())
 
+            song_exp = Gtk.PropertyExpression.new(
+                CoreSong, None, "track-number")
+            song_sorter = Gtk.NumericSorter.new(song_exp)
+            self._model = Gtk.SortListModel.new(filter_model, song_sorter)
             self._model.connect("items-changed", self._on_disc_changed)
 
-            self._get_album_disc(
-                self.props.media, self.props.disc_nr, self._filter_model)
+            self._coregrilo.get_album_disc(
+                self.props.media, self.props.disc_nr, filter_model)
 
         return self._model
 
@@ -82,49 +77,6 @@ class CoreDisc(GObject.GObject):
         with self.freeze_notify():
             duration = 0
             for coresong in model:
-                coresong.props.selected = self._selected
                 duration += coresong.props.duration
 
             self.props.duration = duration
-
-    def _get_album_disc(self, media, discnr, model):
-        album_ids = []
-        model_filter = model
-
-        def _filter_func(core_song):
-            return core_song.props.grlid in album_ids
-
-        def _callback(source, op_id, media, remaining, error):
-            if error:
-                self._log.warning("Error: {}".format(error))
-                return
-
-            if media is None:
-                if sorted(album_ids) == sorted(self._old_album_ids):
-                    return
-                model_filter.set_filter_func(_filter_func)
-                self._old_album_ids = album_ids
-                return
-
-            album_ids.append(media.get_source() + media.get_id())
-
-        self._coregrilo.populate_album_disc_songs(media, discnr, _callback)
-
-    @GObject.Property(
-        type=bool, default=False, flags=GObject.ParamFlags.READWRITE)
-    def selected(self):
-        return self._selected
-
-    @selected.setter  # type: ignore
-    def selected(self, value):
-        if value == self._selected:
-            return
-
-        self._selected = value
-
-        # The model is loaded on-demand, so the first time the model is
-        # returned it can still be empty. This is problem for returning
-        # a selection. Trigger loading of the model here if a selection
-        # is requested, it will trigger the filled model update as
-        # well.
-        self.props.model.items_changed(0, 0, 0)
