@@ -25,8 +25,8 @@
 import weakref
 
 import gi
-gi.require_version('Grl', '0.3')
-from gi.repository import Grl, GLib, GObject
+gi.require_version("Grl", "0.3")
+from gi.repository import Grl, GLib, GObject, Gtk
 
 from gnomemusic.grilowrappers.grlsearchwrapper import GrlSearchWrapper
 from gnomemusic.grilowrappers.grltrackerwrapper import GrlTrackerWrapper
@@ -61,7 +61,6 @@ class CoreGrilo(GObject.GObject):
 
         self._application = application
         self._coremodel = self._application.props.coremodel
-        self._coreselection = application.props.coreselection
         self._log = application.props.log
         self._search_wrappers = {}
         self._thumbnail_sources = []
@@ -178,7 +177,7 @@ class CoreGrilo(GObject.GObject):
         """Get all album by an artist
 
         :param Grl.Media media: A Grilo Media item that represents Artist
-        :param Gfm.FilterListModel filter_model: The model to fill
+        :param Gtk.FilterListModel filter_model: The model to fill
         """
         source = media.get_source()
         self._wrappers[source].get_artist_albums(media, filter_model)
@@ -187,21 +186,22 @@ class CoreGrilo(GObject.GObject):
         """Get all discs from an album
 
         :param Grl.Media media: A Grilo Media item that represents Album
-        :param Gfm.SortListModel disc_model: The model to fill
+        :param Gtk.SortListModel disc_model: The model to fill
         """
         source = media.get_source()
         self._wrappers[source].get_album_discs(media, disc_model)
 
-    def populate_album_disc_songs(self, media, discnr, callback):
+    def get_album_disc(
+            self, media: Grl.Media, discnr: int,
+            model: Gtk.FilterListModel) -> None:
         """Get all songs from an album disc
 
-        :param Grl.Media media: A Grilo Media item that represents Album
+        :param Grl.Media media: An album
         :param int discnr: The disc number
-        :param callback: The callback to call for every song added
+        :param Gtk.FilterListModel model: The model to fill
         """
         source = media.get_source()
-        self._wrappers[source].populate_album_disc_songs(
-            media, discnr, callback)
+        self._wrappers[source].get_album_disc(media, discnr, model)
 
     def writeback(self, media, key):
         """Store the values associated with the key.
