@@ -22,6 +22,7 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
+from __future__ import annotations
 from enum import Enum, IntEnum
 from gettext import gettext as _
 from random import randrange
@@ -30,22 +31,24 @@ import typing
 
 import gi
 gi.require_version('GstPbutils', '1.0')
-from gi.repository import GObject, GstPbutils
+from gi.repository import GLib, GObject, GstPbutils, Gtk
 
 from gnomemusic.coresong import CoreSong
 from gnomemusic.gstplayer import GstPlayer, Playback
 from gnomemusic.widgets.songwidget import SongWidget
-import gnomemusic.utils as utils
+
+if typing.TYPE_CHECKING:
+    from gi.repository import Gio
 
 
 class RepeatMode(Enum):
     """Enum for player repeat mode"""
 
     # Translators: "shuffle" causes tracks to play in random order.
-    SHUFFLE = 0, "media-playlist-shuffle-symbolic", _("Shuffle")
+    NONE = 0, "media-playlist-consecutive-symbolic", _("Shuffle/Repeat Off")
     SONG = 1, "media-playlist-repeat-song-symbolic", _("Repeat Song")
     ALL = 2, "media-playlist-repeat-symbolic", _("Repeat All")
-    NONE = 3, "media-playlist-consecutive-symbolic", _("Shuffle/Repeat Off")
+    SHUFFLE = 3, "media-playlist-shuffle-symbolic", _("Shuffle")
 
     # The type checking is necessary to avoid false positives
     # See: https://github.com/python/mypy/issues/1021
@@ -277,16 +280,16 @@ class PlayerPlaylist(GObject.GObject):
         self._model_recent.set_offset(offset)
 
     def _on_repeat_mode_changed(self, klass, param):
-        def _shuffle_sort(song_a, song_b):
-            return song_a.shuffle_pos < song_b.shuffle_pos
-
         if self.props.repeat_mode == RepeatMode.SHUFFLE:
             for idx, coresong in enumerate(self._model):
                 coresong.update_shuffle_pos()
-            self._model.set_sort_func(
-                utils.wrap_list_store_sort_func(_shuffle_sort))
+
+            song_sorter_exp = Gtk.PropertyExpression.new(
+                CoreSong, None, "shuffle-pos")
+            songs_sorter = Gtk.NumericSorter.new(song_sorter_exp)
+            self._model.set_sorter(songs_sorter)
         elif self.props.repeat_mode in [RepeatMode.NONE, RepeatMode.ALL]:
-            self._model.set_sort_func(None)
+            self._model.set_sorter(None)
 
     def _validate_song(self, coresong):
         # Song is being processed or has already been processed.
@@ -382,6 +385,8 @@ class Player(GObject.GObject):
             "items-changed", self._on_playlist_model_items_changed)
 
         self._settings = application.props.settings
+        self._settings.connect("changed::repeat", self._on_repeat_mode_changed)
+
         self._repeat = RepeatMode(self._settings.get_enum("repeat"))
         self.bind_property(
             'repeat-mode', self._playlist, 'repeat-mode',
@@ -401,8 +406,6 @@ class Player(GObject.GObject):
         self._gst_player.bind_property(
             'state', self, 'state', GObject.BindingFlags.SYNC_CREATE)
 
-        self._lastfm = application.props.lastfm_scrobbler
-
     @GObject.Property(
         type=bool, default=False, flags=GObject.ParamFlags.READABLE)
     def has_next(self):
@@ -438,6 +441,11 @@ class Player(GObject.GObject):
                 and model.get_n_items() == 0):
             self.stop()
 
+    def _on_repeat_mode_changed(
+            self, settings: Gio.Settings, key: str) -> None:
+        repeat_mode = settings.get_enum(key)
+        self.props.repeat_mode = RepeatMode(repeat_mode)
+
     def _on_about_to_finish(self, klass):
         if self.props.has_next:
             self._log.debug("Song is about to finish, loading the next one.")
@@ -570,7 +578,6 @@ class Player(GObject.GObject):
 
         if tick == 0:
             self._new_clock = True
-            self._lastfm.now_playing(current_song)
 
         if self.props.duration == -1.:
             return
@@ -578,21 +585,11 @@ class Player(GObject.GObject):
         position = self._gst_player.props.position
         if position > 0:
             percentage = tick / self.props.duration
-            if (not self._lastfm.props.scrobbled
-                    and self.props.duration > 30.
-                    and (percentage > 0.5 or tick > 4 * 60)):
-                self._lastfm.scrobble(current_song, self._time_stamp)
-
             if (percentage > 0.5
                     and self._new_clock):
                 self._new_clock = False
-                # FIXME: we should not need to update smart
-                # playlists here but removing it may introduce
-                # a bug. So, we keep it for the time being.
-                # FIXME: Not using Playlist class anymore.
-                # playlists.update_all_smart_playlists()
                 current_song.bump_play_count()
-                current_song.set_last_played()
+                current_song.props.last_played = GLib.DateTime.new_now_utc()
 
     @GObject.Property(type=object)
     def repeat_mode(self) -> RepeatMode:
