@@ -24,19 +24,23 @@
 
 from __future__ import annotations
 from enum import IntEnum
-from gettext import gettext as _
 from typing import Optional
 import typing
 
-from gi.repository import GObject, Gtk
+from gi.repository import Adw, GObject, Gtk
+from gettext import gettext as _
 
 from gnomemusic.search import Search
 from gnomemusic.utils import ArtSize
 from gnomemusic.widgets.albumcover import AlbumCover
-from gnomemusic.widgets.albumwidget import AlbumWidget
-from gnomemusic.widgets.headerbar import HeaderBar
-from gnomemusic.widgets.artistalbumswidget import ArtistAlbumsWidget
+from gnomemusic.widgets.albumnavigationpage import AlbumNavigationPage
+from gnomemusic.widgets.albumssearchnavigationpage import (
+    AlbumsSearchNavigationPage)
+from gnomemusic.widgets.artistnavigationpage import ArtistNavigationPage
 from gnomemusic.widgets.artistsearchtile import ArtistSearchTile
+from gnomemusic.widgets.artistssearchnavigationpage import (
+    ArtistsSearchNavigationPage)
+from gnomemusic.widgets.searchheaderbar import SearchHeaderBar
 from gnomemusic.widgets.songwidget import SongWidget
 from gnomemusic.widgets.songwidgetmenu import SongWidgetMenu
 if typing.TYPE_CHECKING:
@@ -46,7 +50,7 @@ if typing.TYPE_CHECKING:
 
 
 @Gtk.Template(resource_path="/org/gnome/Music/ui/SearchView.ui")
-class SearchView(Gtk.Stack):
+class SearchView(Adw.NavigationPage):
     """Gridlike view of search results.
 
     Three sections: artists, albums, songs.
@@ -63,23 +67,19 @@ class SearchView(Gtk.Stack):
         ALBUM = 3
         ARTIST = 4
 
+    search_mode_active = GObject.Property(type=bool, default=False)
     search_state = GObject.Property(type=int, default=Search.State.NONE)
-    selection_mode = GObject.Property(type=bool, default=False)
-    state = GObject.Property(type=int, default=State.MAIN)
-    title = GObject.Property(
-        type=str, default="", flags=GObject.ParamFlags.READABLE)
 
     _album_header = Gtk.Template.Child()
     _album_flowbox = Gtk.Template.Child()
-    _album_all_flowbox = Gtk.Template.Child()
-    _all_search_results = Gtk.Template.Child()
     _artist_header = Gtk.Template.Child()
-    _artist_all_flowbox = Gtk.Template.Child()
     _artist_flowbox = Gtk.Template.Child()
-    _scrolled_album_widget = Gtk.Template.Child()
     _search_results = Gtk.Template.Child()
+    _search_toolbar_view = Gtk.Template.Child()
     _songs_header = Gtk.Template.Child()
     _songs_listbox = Gtk.Template.Child()
+    _stack = Gtk.Template.Child()
+    _status_page = Gtk.Template.Child()
     _view_all_albums = Gtk.Template.Child()
     _view_all_artists = Gtk.Template.Child()
 
@@ -88,7 +88,7 @@ class SearchView(Gtk.Stack):
 
         :param GtkApplication application: The Application object
         """
-        super().__init__(transition_type=Gtk.StackTransitionType.CROSSFADE)
+        super().__init__()
 
         self.props.name = "search"
 
@@ -97,8 +97,22 @@ class SearchView(Gtk.Stack):
         self._model = self._coremodel.props.songs_search
         self._player = self._application.props.player
         self._window = application.props.window
+        self._navigation_view = self._window.props.navigation_view
         self._headerbar = self._window.props.headerbar
 
+        self._search_headerbar = SearchHeaderBar(self._application)
+        self._search_toolbar_view.add_top_bar(self._search_headerbar)
+
+        self.bind_property(
+            "search-state", self._search_headerbar, "search-state",
+            GObject.BindingFlags.BIDIRECTIONAL
+            | GObject.BindingFlags.SYNC_CREATE)
+        self.connect(
+            "notify::search-state", self._on_search_state_changed)
+
+        self._navigation_view.connect(
+            "replaced", self._on_navigation_view_replaced)
+
         self._album_model = self._coremodel.props.albums_search
         self._artist_model = self._coremodel.props.artists_search
 
@@ -110,6 +124,11 @@ class SearchView(Gtk.Stack):
         self._songs_listbox.bind_model(self._model, self._create_song_widget)
         self._on_model_items_changed(self._model, 0, 0, 0)
 
+        self.bind_property(
+            "search_mode_active", self._search_headerbar, "search_mode_active",
+            GObject.BindingFlags.BIDIRECTIONAL
+            | GObject.BindingFlags.SYNC_CREATE)
+
         self._album_slice.connect_after(
             "items-changed", self._on_album_model_items_changed)
         self._album_flowbox.bind_model(
@@ -122,22 +141,10 @@ class SearchView(Gtk.Stack):
         self._artist_flowbox.bind_model(
             self._artist_slice, self._create_artist_widget)
 
-        self.connect("notify::selection-mode", self._on_selection_mode_changed)
-
-        self.bind_property(
-            'selection-mode', self._window, 'selection-mode',
-            GObject.BindingFlags.BIDIRECTIONAL)
-
-        self._album_widget = AlbumWidget(self._application)
-        self._album_widget.bind_property(
-            "selection-mode", self, "selection-mode",
-            GObject.BindingFlags.BIDIRECTIONAL)
-        viewport = self._scrolled_album_widget.get_first_child()
-        viewport.set_child(self._album_widget)
-
         self._scrolled_artist_window: Optional[Gtk.ScrolledWindow] = None
 
-        self._search_mode_active = False
+        self._set_empty_status_page()
+        self._stack.props.visible_child_name = "status"
 
     def _core_filter(self, coreitem, coremodel, nr_items):
         if coremodel.get_n_items() <= 5:
@@ -155,39 +162,17 @@ class SearchView(Gtk.Stack):
         song_widget.props.menu = SongWidgetMenu(
             self._application, song_widget, coresong)
 
-        self.bind_property(
-            "selection-mode", song_widget, "selection-mode",
-            GObject.BindingFlags.BIDIRECTIONAL
-            | GObject.BindingFlags.SYNC_CREATE)
-
         return song_widget
 
     def _create_album_cover(self, corealbum: CoreAlbum) -> AlbumCover:
         album_cover = AlbumCover(corealbum)
         album_cover.retrieve()
 
-        self.bind_property(
-            "selection-mode", album_cover, "selection-mode",
-            GObject.BindingFlags.SYNC_CREATE
-            | GObject.BindingFlags.BIDIRECTIONAL)
-
-        # NOTE: Adding SYNC_CREATE here will trigger all the nested
-        # models to be created. This will slow down initial start,
-        # but will improve initial 'select all' speed.
-        album_cover.bind_property(
-            "selected", corealbum, "selected",
-            GObject.BindingFlags.BIDIRECTIONAL)
-
         return album_cover
 
     def _create_artist_widget(self, coreartist):
         artist_tile = ArtistSearchTile(coreartist)
 
-        self.bind_property(
-            "selection-mode", artist_tile, "selection-mode",
-            GObject.BindingFlags.SYNC_CREATE
-            | GObject.BindingFlags.BIDIRECTIONAL)
-
         return artist_tile
 
     def _on_album_model_items_changed(self, model, position, removed, added):
@@ -215,6 +200,33 @@ class SearchView(Gtk.Stack):
         self._songs_listbox.props.visible = items_found
         self._check_visibility()
 
+    def _on_navigation_view_replaced(self, view: Adw.NavigationView) -> None:
+        self._search_headerbar.clear_entry()
+
+    def _on_search_state_changed(
+            self, searchview: SearchView, state: GObject.GParamInt) -> None:
+        state = self.props.search_state
+
+        if state == Search.State.RESULT:
+            self._stack.props.visible_child_name = "main"
+        elif state == Search.State.NONE:
+            self._set_empty_status_page()
+            self._stack.props.visible_child_name = "status"
+        elif state == Search.State.NO_RESULT:
+            self._set_no_result_status_page()
+            self._stack.props.visible_child_name = "status"
+
+    def _set_empty_status_page(self) -> None:
+        self._status_page.props.title = _("No Search Started")
+        self._status_page.props.description = _(
+            "Use the searchbar to start searching for "
+            "albums, artists or songs")
+
+    def _set_no_result_status_page(self) -> None:
+        self._status_page.props.title = _("No Results Found")
+        self._status_page.props.description = _(
+            "Try a different search")
+
     def _check_visibility(self):
         if not self.props.search_mode_active:
             return
@@ -231,11 +243,6 @@ class SearchView(Gtk.Stack):
     def _song_activated(
             self, list_box: Gtk.ListBox, song_widget: SongWidget) -> bool:
         coresong = song_widget.props.coresong
-        if self.props.selection_mode:
-            selection_state = coresong.props.selected
-            song_widget.props.selected = not selection_state
-            coresong.props.selected = not selection_state
-            return True
 
         self._coremodel.props.active_core_object = coresong
         self._player.play(coresong)
@@ -254,148 +261,23 @@ class SearchView(Gtk.Stack):
     @Gtk.Template.Callback()
     def _on_album_activated(self, widget, child, user_data=None):
         corealbum = child.props.corealbum
-        if self.props.selection_mode:
-            corealbum.props.selected = not corealbum.props.selected
-            return
-
-        # Update and display the album widget if not in selection mode
-        self._album_widget.props.corealbum = corealbum
-
-        self.props.state = SearchView.State.ALBUM
-        self._headerbar.props.state = HeaderBar.State.SEARCH
-        self._headerbar.set_label_title(
-            corealbum.props.title, corealbum.props.artist)
-
-        self.set_visible_child(self._scrolled_album_widget)
-        self.props.search_mode_active = False
+        album_page = AlbumNavigationPage(self._application, corealbum)
+        self._navigation_view.push(album_page)
 
     @Gtk.Template.Callback()
     def _on_artist_activated(self, widget, child, user_data=None):
         coreartist = child.props.coreartist
-        if self.props.selection_mode:
-            return
-
-        artist_albums_widget = ArtistAlbumsWidget(self._application)
-        artist_albums_widget.props.coreartist = coreartist
-        # FIXME: Recreating a view here. Alternate solution is used
-        # in AlbumsView: one view created and an update function.
-        # Settle on one design.
-        self._scrolled_artist_window = Gtk.ScrolledWindow()
-        self._scrolled_artist_window.props.child = artist_albums_widget
-        self._scrolled_artist_window.props.visible = True
-        self.add_child(self._scrolled_artist_window)
-        artist_albums_widget.show()
-
-        self.bind_property(
-            "selection-mode", artist_albums_widget, "selection-mode",
-            GObject.BindingFlags.BIDIRECTIONAL)
-
-        self.props.state = SearchView.State.ARTIST
-        self._headerbar.props.state = HeaderBar.State.SEARCH
-        self._headerbar.set_label_title(coreartist.props.artist, "")
-
-        self.set_visible_child(self._scrolled_artist_window)
-        self.props.search_mode_active = False
+        artist_page = ArtistNavigationPage(self._application, coreartist)
+        self._navigation_view.push(artist_page)
 
     @Gtk.Template.Callback()
     def _on_all_artists_clicked(self, widget, user_data=None):
-        self.props.state = SearchView.State.ALL_ARTISTS
-        self._headerbar.props.state = HeaderBar.State.SEARCH
-        self._headerbar.set_label_title(_("Artists Results"), "")
-
-        self._artist_all_flowbox.props.visible = True
-        self._album_all_flowbox.props.visible = False
-        self._artist_all_flowbox.bind_model(
-            self._artist_model, self._create_artist_widget)
-
-        self.props.visible_child = self._all_search_results
-        self.props.search_mode_active = False
+        all_artists_page = ArtistsSearchNavigationPage(
+            self._application, self._artist_model)
+        self._navigation_view.push(all_artists_page)
 
     @Gtk.Template.Callback()
     def _on_all_albums_clicked(self, widget, user_data=None):
-        self.props.state = SearchView.State.ALL_ALBUMS
-        self._headerbar.props.state = HeaderBar.State.SEARCH
-        self._headerbar.set_label_title(_("Albums Results"), "")
-
-        self._artist_all_flowbox.props.visible = False
-        self._album_all_flowbox.props.visible = True
-        self._album_all_flowbox.bind_model(
-            self._album_model, self._create_album_cover)
-
-        self.props.visible_child = self._all_search_results
-        self.props.search_mode_active = False
-
-    def _select_all(self, value: bool) -> None:
-        if self.props.state == SearchView.State.MAIN:
-            with self._model.freeze_notify():
-                for coresong in self._model:
-                    coresong.props.selected = value
-                for corealbum in self._album_model:
-                    corealbum.props.selected = value
-                for coreartist in self._artist_model:
-                    coreartist.props.selected = value
-        elif self.props.state == SearchView.State.ALL_ALBUMS:
-            with self._model.freeze_notify():
-                for corealbum in self._album_model:
-                    corealbum.props.selected = value
-        elif self.props.state == SearchView.State.ALL_ARTISTS:
-            with self._model.freeze_notify():
-                for corealbum in self._album_model:
-                    corealbum.props.selected = value
-        elif self.props.state == SearchView.State.ALBUM:
-            view = self._album_widget
-            if value is True:
-                view.select_all()
-            else:
-                view.deselect_all()
-        elif self.props.state == SearchView.State.ARTIST:
-            view = self.get_visible_child().get_child().get_child()
-            if value is True:
-                view.select_all()
-            else:
-                view.deselect_all()
-
-    def select_all(self):
-        self._select_all(True)
-
-    def deselect_all(self):
-        self._select_all(False)
-
-    def _back_button_clicked(self, widget, data=None):
-        if self.get_visible_child() == self._search_results:
-            return
-        elif self.get_visible_child() == self._scrolled_artist_window:
-            self.remove(self._scrolled_artist_window)
-
-        self.set_visible_child(self._search_results)
-        self.props.search_mode_active = True
-        self.props.state = SearchView.State.MAIN
-        self._headerbar.props.state = HeaderBar.State.MAIN
-
-    def _on_selection_mode_changed(self, widget, data=None):
-        if (not self.props.selection_mode
-                and self.get_parent().get_visible_child() == self):
-            self.deselect_all()
-
-    @GObject.Property(type=bool, default=False)
-    def search_mode_active(self):
-        """Get search mode status.
-
-        :returns: the search mode status
-        :rtype: bool
-        """
-        return self._search_mode_active
-
-    @search_mode_active.setter  # type: ignore
-    def search_mode_active(self, value):
-        """Set search mode status.
-
-        :param bool mode: new search mode
-        """
-        # FIXME: search_mode_active should not change search_state.
-        # This is necessary because Search state cannot interact with
-        # the child views.
-        self._search_mode_active = value
-        if (not self._search_mode_active
-                and self.get_visible_child() == self._search_results):
-            self.props.search_state = Search.State.NONE
+        all_albums_page = AlbumsSearchNavigationPage(
+            self._application, self._album_model)
+        self._navigation_view.push(all_albums_page)
